<?php

if (! defined ( 'BASEPATH' ))
	exit ( 'No direct script access allowed' );
/**
 * CodeIgniter
 *
 * An open source application development framework for PHP 5.1.6 or newer
 *
 * @package CodeIgniter
 * @author EllisLab Dev Team
 * @copyright Copyright (c) 2008 - 2014, EllisLab, Inc.
 * @copyright Copyright (c) 2014 - 2015, British Columbia Institute of Technology (http://bcit.ca/)
 * @license http://codeigniter.com/user_guide/license.html
 * @link http://codeigniter.com
 * @since Version 2.1.2
 * @filesource
 *
 */
	
// ------------------------------------------------------------------------

/**
 * PDO Forge Class
 *
 * @category Database
 * @author EllisLab Dev Team
 * @link http://codeigniter.com/database/
 */
class CI_DB_pdo_forge extends CI_DB_forge {
	
	/**
	 * Create database
	 *
	 * @access private
	 * @param
	 *        	string the database name
	 * @return bool
	 */
	function _create_database() {
		// PDO has no "create database" command since it's
		// designed to connect to an existing database
		if ($this->db->db_debug) {
			return $this->db->display_error ( 'db_unsuported_feature' );
		}
		return FALSE;
	}
	
	// --------------------------------------------------------------------
	
	/**
	 * Drop database
	 *
	 * @access private
	 * @param
	 *        	string the database name
	 * @return bool
	 */
	function _drop_database($name) {
		// PDO has no "drop database" command since it's
		// designed to connect to an existing database
		if ($this->db->db_debug) {
			return $this->db->display_error ( 'db_unsuported_feature' );
		}
		return FALSE;
	}
	
	// --------------------------------------------------------------------
	
	/**
	 * Create Table
	 *
	 * @access private
	 * @param
	 *        	string the table name
	 * @param
	 *        	array the fields
	 * @param
	 *        	mixed primary key(s)
	 * @param
	 *        	mixed key(s)
	 * @param
	 *        	boolean should 'IF NOT EXISTS' be added to the SQL
	 * @return bool
	 */
	function _create_table($table, $fields, $primary_keys, $keys, $if_not_exists) {
		$sql = 'CREATE TABLE ';
		
		if ($if_not_exists === TRUE) {
			$sql .= 'IF NOT EXISTS ';
		}
		
		$sql .= $this->db->_escape_identifiers ( $table ) . " (";
		$current_field_count = 0;
		
		foreach ( $fields as $field => $attributes ) {
			// Numeric field names aren't allowed in databases, so if the key is
			// numeric, we know it was assigned by PHP and the developer manually
			// entered the field information, so we'll simply add it to the list
			if (is_numeric ( $field )) {
				$sql .= "\n\t$attributes";
			} else {
				$attributes = array_change_key_case ( $attributes, CASE_UPPER );
				
				$sql .= "\n\t" . $this->db->_protect_identifiers ( $field );
				
				$sql .= ' ' . $attributes ['TYPE'];
				
				if (array_key_exists ( 'CONSTRAINT', $attributes )) {
					$sql .= '(' . $attributes ['CONSTRAINT'] . ')';
				}
				
				if (array_key_exists ( 'UNSIGNED', $attributes ) && $attributes ['UNSIGNED'] === TRUE) {
					$sql .= ' UNSIGNED';
				}
				
				if (array_key_exists ( 'DEFAULT', $attributes )) {
					$sql .= ' DEFAULT \'' . $attributes ['DEFAULT'] . '\'';
				}
				
				if (array_key_exists ( 'NULL', $attributes ) && $attributes ['NULL'] === TRUE) {
					$sql .= ' NULL';
				} else {
					$sql .= ' NOT NULL';
				}
				
				if (array_key_exists ( 'AUTO_INCREMENT', $attributes ) && $attributes ['AUTO_INCREMENT'] === TRUE) {
					$sql .= ' AUTO_INCREMENT';
				}
			}
			
			// don't add a comma on the end of the last field
			if (++ $current_field_count < count ( $fields )) {
				$sql .= ',';
			}
		}
		
		if (count ( $primary_keys ) > 0) {
			$primary_keys = $this->db->_protect_identifiers ( $primary_keys );
			$sql .= ",\n\tPRIMARY KEY (" . implode ( ', ', $primary_keys ) . ")";
		}
		
		if (is_array ( $keys ) && count ( $keys ) > 0) {
			foreach ( $keys as $key ) {
				if (is_array ( $key )) {
					$key = $this->db->_protect_identifiers ( $key );
				} else {
					$key = array (
							$this->db->_protect_identifiers ( $key ) 
					);
				}
				
				$sql .= ",\n\tFOREIGN KEY (" . implode ( ', ', $key ) . ")";
			}
		}
		
		$sql .= "\n)";
		
		return $sql;
	}
	
	// --------------------------------------------------------------------
	
	/**
	 * Drop Table
	 *
	 * @access private
	 * @return bool
	 */
	function _drop_table($table) {
		// Not a supported PDO feature
		if ($this->db->db_debug) {
			return $this->db->display_error ( 'db_unsuported_feature' );
		}
		return FALSE;
	}
	
	// --------------------------------------------------------------------
	
	/**
	 * Alter table query
	 *
	 * Generates a platform-specific query so that a table can be altered
	 * Called by add_column(), drop_column(), and column_alter(),
	 *
	 * @access private
	 * @param
	 *        	string the ALTER type (ADD, DROP, CHANGE)
	 * @param
	 *        	string the column name
	 * @param
	 *        	string the table name
	 * @param
	 *        	string the column definition
	 * @param
	 *        	string the default value
	 * @param
	 *        	boolean should 'NOT NULL' be added
	 * @param
	 *        	string the field after which we should add the new field
	 * @return object
	 */
	function _alter_table($alter_type, $table, $column_name, $column_definition = '', $default_value = '', $null = '', $after_field = '') {
		$sql = 'ALTER TABLE ' . $this->db->_protect_identifiers ( $table ) . " $alter_type " . $this->db->_protect_identifiers ( $column_name );
		
		// DROP has everything it needs now.
		if ($alter_type == 'DROP') {
			return $sql;
		}
		
		$sql .= " $column_definition";
		
		if ($default_value != '') {
			$sql .= " DEFAULT \"$default_value\"";
		}
		
		if ($null === NULL) {
			$sql .= ' NULL';
		} else {
			$sql .= ' NOT NULL';
		}
		
		if ($after_field != '') {
			$sql .= ' AFTER ' . $this->db->_protect_identifiers ( $after_field );
		}
		
		return $sql;
	}
	
	// --------------------------------------------------------------------
	
	/**
	 * Rename a table
	 *
	 * Generates a platform-specific query so that a table can be renamed
	 *
	 * @access private
	 * @param
	 *        	string the old table name
	 * @param
	 *        	string the new table name
	 * @return string
	 */
	function _rename_table($table_name, $new_table_name) {
		$sql = 'ALTER TABLE ' . $this->db->_protect_identifiers ( $table_name ) . " RENAME TO " . $this->db->_protect_identifiers ( $new_table_name );
		return $sql;
	}
}

/* End of file pdo_forge.php */
/* Location: ./system/database/drivers/pdo/pdo_forge.php */